from dataclasses import dataclass


class BrokenLink(Exception):
    # Ignore these
    pass


@dataclass
class Link:
    """Dataclass to hold the contents required to form a complete Link."""

    image: bool
    text: str
    target: str
    anchor: str
    title: str

    # Render as a complete MD compatible link
    def render(self, not_found=False):
        img = "!" if self.image else ""
        anchor = f"#{self.anchor}" if self.anchor else ""
        title = f' "{self.title}"' if self.title else ""
        broken = "{: .ezlinks_not_found}" if not_found else ""
        if not_found:
            img = ""
        return f"{img}[{self.text}]({self.target}{anchor}{title}){broken}"


@dataclass
class LinksOptions:
    """Dataclass to hold typed options from the configuration."""

    wikilinks: bool
    warn_ambiguities: bool
    reference_links: bool



import logging
from typing import List

from mkdocs.plugins import BasePlugin
from mkdocs.config import config_options
from mkdocs.structure.files import File


from .file_mapper import FileMapper
from .replacer import LinksReplacer
from .scanners.md_link_scanner import MdLinkScanner
from .scanners.wiki_link_scanner import WikiLinkScanner
from .scanners.reference_link_scanner import ReferenceLinkScanner
from .types import LinksOptions

LOGGER = logging.getLogger(f"mkdocs.plugins.{__name__}")


class LinksPlugin(BasePlugin):
    config_scheme = (
        ("wikilinks", config_options.Type(bool, default=True)),
        ("warn_ambiguities", config_options.Type(bool, default=False)),
        ("reference_links", config_options.Type(bool, default=False)),
    )

    def init(self, config):
        self.replacer = LinksReplacer(
            root=config["docs_dir"],
            file_map=self.file_mapper,
            use_directory_urls=config["use_directory_urls"],
            options=LinksOptions(**self.config),
            logger=LOGGER,
        )

        self.replacer.add_scanner(MdLinkScanner())
        if self.config["wikilinks"]:
            self.replacer.add_scanner(WikiLinkScanner())

        if self.config["reference_links"]:
            self.replacer.add_scanner(ReferenceLinkScanner())

        # Compile the regex once
        self.replacer.compile()

    # Build a fast lookup of all files (by file name)
    def on_files(self, files: List[File], config):
        self.file_mapper = FileMapper(
            options=LinksOptions(**self.config),
            root=config["docs_dir"],
            files=files,
            logger=LOGGER,
        )

        # After the file map has been built, initialize what we can that will
        # remain static
        self.init(config)

    def on_page_markdown(self, markdown, page, config, **kwargs):
        return self.replacer.replace(page.file.src_uri, markdown)
